﻿import event.*;
import util.*;

/**
 * Generates random events.
 */
class util.Randomizer {
  // The event listeners that will receive events.
  private var listenerList:EventListenerList;
  // The time in milliseconds between checks for a random event.
  private var randInterval:Number;
  // The time of the last random event.
  private var lastEventTime:Date;

  /**
   * Constructor
   *
   * @param  interval  The time in milliseconds to wait between 
   *                   checks for a random event.
   * @param  odds      The likelihood an event will be triggered at
   *                   each check. An event has a 1 in odds chance of
   *                   being triggered.
   */
  public function Randomizer (interval:Number, odds:Number) {
    // Create an EventListenerList to manage listeners.
    listenerList = new EventListenerList();

    // Initialize the time of the most recent event.
    lastEventTime = new Date();

    // Start checking for events.
    start(interval, odds);
  }

  /**
   * Register an object to receive random events.
   */
  public function addRandomizerListener (l:RandomizerListener):Boolean {
    return listenerList.addObj(l);
  }
  
  /**
   * Unregister an event listener object.
   */
  public function removeRandomizerListener (l:RandomizerListener):Boolean {
    return listenerList.removeObj(l);
  }

  /**
   * Start an interval to check for random events.
   *
   * @param  interval  The time in milliseconds to wait between 
   *                   checks for a random event.
   * @param  odds      The likelihood an event will be triggered at
   *                   each check. An event has a 1 in odds chance of
   *                   being triggered.
   */
  public function start (interval:Number, odds:Number):Boolean {
    if (interval > 0 && odds > 1) {
      // Call this.check() every interval milliseconds.
      randInterval = setInterval(this, "check", interval, odds);
      return true;
    }
    return false;
  }

  /**
   * Stop checking for random events.
   */
  public function stop ():Void {
    clearInterval(randInterval);
  }

  /**
   * Restart the event-checking interval, possibly with new odds.
   *
   * @param  interval  The time in milliseconds to wait between 
   *                   checks for a random event.
   * @param  odds      The likelihood an event will be triggered at
   *                   each check. An event has a 1 in odds chance of
   *                   being triggered.
   */
  public function restart (interval:Number, odds:Number):Void {
    stop();
    start(interval, odds);
  } 

  /**
   * Checks to see if a random event occurs, based on the
   * current odds.
   *
   * @param  odds      The likelihood an event will be triggered at
   *                   each check. An event has a 1 in odds chance of
   *                   being triggered.
   */
  private function check (odds:Number):Void {
    // Local variables.
    var rand:Number = Math.floor(Math.random() * odds);
    var now:Date = new Date();
    var elapsed:Number;

    // If the random event occurs...
    if (rand == 0) {
      // Determine the elapsed time since the last event.
      elapsed = now.getTime() - lastEventTime.getTime();
      lastEventTime = now;

      // Fire the event.
      fireOnRandomAction(elapsed);
    }
  }

  /**
   * Invokes onRandomAction on all listeners.
   *
   * @param  elapsed   The amount of time since the last event notification.
   */
  private function fireOnRandomAction (elapsed:Number):Void {
    // Create an object to describe the event.
    var e:RandomizerEvent = new RandomizerEvent(this, elapsed);
    // Get a list of the current event listeners.
    var listeners:Array = listenerList.getListeners();

    // Broadcast the event to all listeners.
    for (var i:Number = 0; i < listeners.length; i++) {
      listeners[i].onRandomAction(e);
    }
  }
}